@@ -98,8 +98,9 @@ string IIPResponse::formatResponse() {
    */
   string response;
   if( error.length() ){
-    response = server + eof + "Cache-Control: no-cache" + eof + mimeType + eof +
-      "Status: 400 Bad Request" + eof +
+    response = server + eof + "Cache-Control: no-cache" + eof + mimeType + eof;
+    if( !cors.empty() ) response += cors + eof;
+    response += "Status: 400 Bad Request" + eof +
       "Content-Disposition: inline;filename=\"IIPisAMadGameClosedToOurUnderstanding.netfpx\"" +
       eof + eof + error;
   }
@@ -114,13 +115,13 @@ string IIPResponse::formatResponse() {
 
 
 
-string IIPResponse::getAdvert( const string& version ){
+string IIPResponse::getAdvert(){
 
   string advert = server + eof + "Content-Type: text/html" + eof;
   advert += "Status: 400 Bad Request" + eof;
   advert += "Content-Disposition: inline;filename=\"iipsrv.html\"" + eof + eof;
   advert += "<!DOCTYPE html><html lang=\"en\"><head><meta charset=\"utf-8\"/><title>IIPImage Server</title><meta name=\"DC.creator\" content=\"Ruven Pillay &lt;ruven@users.sourceforge.net&gt;\"/><meta name=\"DC.title\" content=\"IIPImage Server\"/><meta name=\"DC.source\" content=\"http://iipimage.sourceforge.net\"/></head><body style=\"font-family:Helvetica,sans-serif; margin:4em\"><center><h1>IIPImage Server</h1><h2>Version "
-    + version +
+    + string( VERSION ) +
     "</h2><br/><h3>Project Home Page: <a href=\"http://iipimage.sourceforge.net\">http://iipimage.sourceforge.net</a></h3><br/><h4>by<br/>Ruven Pillay</h4></center></body></html>";
 
   return advert;
