@@ -1,7 +1,7 @@
 /*
     IIP Response Handler Class
 
-    Copyright (C) 2003-2015 Ruven Pillay.
+    Copyright (C) 2003-2018 Ruven Pillay.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -27,7 +27,7 @@
 #endif
 
 // Fix missing snprintf in Windows
-#if _MSC_VER
+#if defined _MSC_VER && _MSC_VER<1900
 #define snprintf _snprintf
 #endif
 
@@ -67,7 +67,7 @@ class IIPResponse{
 
 
   /// Set the Last Modified header
-  /** @param m Last modifed date as a HTTP RFC 1123 formatted timestamp */
+  /** @param m Last modified date as a HTTP RFC 1123 formatted timestamp */
   void setLastModified( const std::string& m ) { modified = "Last-Modified: " + m; };
 
 
@@ -111,8 +111,13 @@ class IIPResponse{
 
 
   /// Set CORS setting
-  /** @param cors setting */
-  void setCORS( const std::string& c ){ if(!c.empty()) cors = "Access-Control-Allow-Origin: " + c; };
+  /** @param c setting */
+  void setCORS( const std::string& c ){
+    if(!c.empty()){
+      cors = "Access-Control-Allow-Origin: " + c + eof +
+	"Access-Control-Allow-Headers: X-Requested-With";
+    }
+  };
 
 
   /// Get CORS setting
@@ -120,7 +125,7 @@ class IIPResponse{
 
 
   /// Set Cache-Control value
-  /** @param Cache-Control setting */
+  /** @param c Cache-Control setting */
   void setCacheControl( const std::string& c ){ cacheControl = "Cache-Control: " + c; };
 
 
@@ -155,8 +160,8 @@ class IIPResponse{
 
 
   /// Display our advertising banner ;-)
-  /** @param version server version */
-  std::string getAdvert( const std::string& version );
+  /** @return HTML string */
+  std::string getAdvert();
 
 
 };
