@@ -1,6 +1,6 @@
 /*  JPEG class wrapper to ijg jpeg library
 
-    Copyright (C) 2000-2015 Ruven Pillay.
+    Copyright (C) 2000-2019 Ruven Pillay.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -25,7 +25,25 @@
 using namespace std;
 
 
-#define MX 32768
+#define MX 65535
+
+
+/* Since an ICC profile can be larger than the maximum size of a JPEG marker
+ * (64K), we need provisions to split it into multiple markers.  The format
+ * defined by the ICC specifies one or more APP2 markers containing the
+ * following data:
+ *      Identifying string      ASCII "ICC_PROFILE\0"  (12 bytes)
+ *      Marker sequence number  1 for first APP2, 2 for next, etc (1 byte)
+ *      Number of markers       Total number of APP2's used (1 byte)
+ *      Profile data            (remainder of APP2 data)
+ * Decoders should use the marker sequence numbers to reassemble the profile,
+ * rather than assuming that the APP2 markers appear in the correct sequence.
+ */
+#define ICC_MARKER  (JPEG_APP0 + 2)     /* JPEG marker code for ICC */
+#define ICC_OVERHEAD_LEN  14            /* size of non-profile data in APP2 */
+#define MAX_BYTES_IN_MARKER  65533      /* maximum data len of a JPEG marker */
+#define MAX_DATA_BYTES_IN_MARKER  (MAX_BYTES_IN_MARKER - ICC_OVERHEAD_LEN)
+
 
 
 /* My version of the JPEG error_exit function. We want to pass control back
@@ -89,7 +107,7 @@ iip_init_destination (j_compress_ptr cinfo)
   */
   mx += MX;
 
-  /* Allocate the output buffer --- it will be released when done with image   
+  /* Allocate the output buffer --- it will be released when done with image
   dest->buffer = (JOCTET *)
     (*cinfo->mem->alloc_small) ( (j_common_ptr) cinfo, JPOOL_IMAGE,
 				 mx * sizeof(JOCTET) );
@@ -158,7 +176,7 @@ void iip_term_destination( j_compress_ptr cinfo )
 
 
 
-void JPEGCompressor::InitCompression( const RawTile& rawtile, unsigned int strip_height ) throw (string)
+void JPEGCompressor::InitCompression( const RawTile& rawtile, unsigned int strip_height )
 {
   // Do some initialisation
   dest = &dest_mgr;
@@ -182,7 +200,7 @@ void JPEGCompressor::InitCompression( const RawTile& rawtile, unsigned int strip
   cinfo.err = jpeg_std_error( &jerr );
 
 
-  // Overide the error_exit function with our own.
+  // Override the error_exit function with our own.
   // Hmmm, we have to do this assignment in C due to the strong type checking of C++
   //  or something like that. So, we use an extern "C" function declared at the top
   //  of this file and pass our arguments through this. I'm sure there's a better
@@ -246,6 +264,12 @@ void JPEGCompressor::InitCompression( const RawTile& rawtile, unsigned int strip
   // Add an identifying comment
   jpeg_write_marker( &cinfo, JPEG_COM, (const JOCTET*) "Generated by IIPImage", 21 );
 
+  // Embed ICC profile if one is supplied
+  writeICCProfile();
+
+  // Add XMP metadata
+  writeXMPMetadata();
+
 }
 
 
@@ -254,7 +278,7 @@ void JPEGCompressor::InitCompression( const RawTile& rawtile, unsigned int strip
   We use a separate tile_height from the predefined strip_height because
   the tile height for the final row can be different
  */
-unsigned int JPEGCompressor::CompressStrip( unsigned char* input, unsigned char* output, unsigned int tile_height ) throw (string)
+unsigned int JPEGCompressor::CompressStrip( unsigned char* input, unsigned char* output, unsigned int tile_height )
 {
   JSAMPROW row[1];
   int row_stride = width * channels;
@@ -286,7 +310,7 @@ unsigned int JPEGCompressor::CompressStrip( unsigned char* input, unsigned char*
 
 
 
-unsigned int JPEGCompressor::Finish( unsigned char* output ) throw (string)
+unsigned int JPEGCompressor::Finish( unsigned char* output )
 {
   dest->source = output;
 
@@ -305,12 +329,10 @@ unsigned int JPEGCompressor::Finish( unsigned char* output ) throw (string)
 
 
 
-int JPEGCompressor::Compress( RawTile& rawtile ) throw (string)
+unsigned int JPEGCompressor::Compress( RawTile& rawtile )
 {
-
   // Do some initialisation
   data = (unsigned char*) rawtile.data;
-  struct jpeg_compress_struct cinfo;
   struct jpeg_error_mgr jerr;
   iip_destination_mgr dest_mgr;
   iip_dest_ptr dest = &dest_mgr;
@@ -334,7 +356,7 @@ int JPEGCompressor::Compress( RawTile& rawtile ) throw (string)
   cinfo.err = jpeg_std_error( &jerr );
 
 
-  // Overide the error_exit function with our own.
+  // Override the error_exit function with our own.
   // Hmmm, we have to do this assignment in C due to the strong type checking of C++
   //  or something like that. So, we use an extern "C" function declared at the top
   //  of this file and pass our arguments through this. I'm sure there's a better
@@ -352,7 +374,6 @@ int JPEGCompressor::Compress( RawTile& rawtile ) throw (string)
    * manager serially with the same JPEG object, because their private object
    * sizes may be different.  Caveat programmer.
    */
-
   if( !cinfo.dest ){
 
     // first time for this JPEG object?
@@ -389,13 +410,18 @@ int JPEGCompressor::Compress( RawTile& rawtile ) throw (string)
 
   // Add an identifying comment
   jpeg_write_marker( &cinfo, JPEG_COM, (const JOCTET*) "Generated by IIPImage", 21 );
-  //jpeg_write_marker( &cinfo, JPEG_APP0+1, (const JOCTET*) , )
+
+  // Embed ICC profile if one is supplied
+  writeICCProfile();
+
+  // Add XMP metadata
+  writeXMPMetadata();
+
 
   // Send the tile data
   unsigned int y;
   int row_stride = width * channels;
 
-
   // Try to pass the whole image array at once if it is less than 512x512 pixels:
   // Should be faster than scanlines.
   if( (row_stride * height) <= (512*512*channels) ){
@@ -448,6 +474,90 @@ int JPEGCompressor::Compress( RawTile& rawtile ) throw (string)
 
 
 
-void JPEGCompressor::addMetadata( const string& metadata ){
-  jpeg_write_marker( &cinfo, JPEG_APP0, (const JOCTET*) metadata.c_str(), metadata.size() );
+// Write ICC profile into JPEG header if profile has been set
+// Function *must* be called AFTER calling jpeg_start_compress() and BEFORE
+// the first call to jpeg_write_scanlines().
+// (This ordering ensures that the APP2 marker(s) will appear after the
+// SOI and JFIF or Adobe markers, but before all else.)
+//
+// The ICC write function is based on an implementation by the Independent JPEG Group
+// See the copyright notice in COPYING.ijg for details
+void JPEGCompressor::writeICCProfile()
+{
+  unsigned int num_markers;     // total number of markers we'll write
+  int cur_marker = 1;           // per spec, counting starts at 1
+  unsigned int length;          // number of bytes to write in this marker
+
+  if( icc.size() == 0 ) return;
+
+  unsigned int icc_data_len = icc.size();
+  const char* icc_data_ptr = icc.c_str();
+
+  // Calculate the number of markers we'll need, rounding up of course
+  num_markers = icc_data_len / MAX_DATA_BYTES_IN_MARKER;
+  if( num_markers * MAX_DATA_BYTES_IN_MARKER != icc_data_len ){
+    num_markers++;
+  }
+
+  while( icc_data_len > 0 ){
+
+    // Length of profile to put in this marker
+    length = icc_data_len;
+    if( length > MAX_DATA_BYTES_IN_MARKER ){
+      length = MAX_DATA_BYTES_IN_MARKER;
+    }
+    icc_data_len -= length;
+
+    // Write the JPEG marker header (APP2 code and marker length)
+    jpeg_write_m_header( &cinfo, ICC_MARKER,
+			 (unsigned int) (length + ICC_OVERHEAD_LEN) );
+
+    // Write the marker identifying string "ICC_PROFILE" (null-terminated).
+    // We code it in this less-than-transparent way so that the code works
+    // even if the local character set is not ASCII.
+    jpeg_write_m_byte(&cinfo, 0x49);
+    jpeg_write_m_byte(&cinfo, 0x43);
+    jpeg_write_m_byte(&cinfo, 0x43);
+    jpeg_write_m_byte(&cinfo, 0x5F);
+    jpeg_write_m_byte(&cinfo, 0x50);
+    jpeg_write_m_byte(&cinfo, 0x52);
+    jpeg_write_m_byte(&cinfo, 0x4F);
+    jpeg_write_m_byte(&cinfo, 0x46);
+    jpeg_write_m_byte(&cinfo, 0x49);
+    jpeg_write_m_byte(&cinfo, 0x4C);
+    jpeg_write_m_byte(&cinfo, 0x45);
+    jpeg_write_m_byte(&cinfo, 0x0);
+
+    // Add the sequencing info
+    jpeg_write_m_byte( &cinfo, cur_marker );
+    jpeg_write_m_byte( &cinfo, (int) num_markers );
+
+    // Add the profile data
+    while( length-- ){
+      jpeg_write_m_byte(&cinfo, *icc_data_ptr);
+      icc_data_ptr++;
+    }
+    cur_marker++;
+  }
+}
+
+
+
+void JPEGCompressor::writeXMPMetadata()
+{
+  // Make sure our XMP data has a valid size (namespace prefix is 29 bytes)
+  if( xmp.size()==0 || xmp.size()>(65536-29) ) return;
+
+  // The XMP data in a JPEG stream needs to be prefixed with a zero-terminated ID string
+  // ref http://www.adobe.com/content/dam/Adobe/en/devnet/xmp/pdfs/cs6/XMPSpecificationPart3.pdf (pp13-14)
+  // so have to copy it into a larger buffer
+  char xmpstr[65536];	// per spec, XMP max size is 65502; namespace prefix/id is 29 bytes
+
+  // '0' should be 0, but snprintf is too smart...
+  //  snprintf( xmpstr, 65536, "http://ns.adobe.com/xap/1.0/%c%s", '0', xmp.c_str() );
+  snprintf( xmpstr, 65536, "http://ns.adobe.com/xap/1.0/%c%s", '\0', xmp.c_str() );
+  //  xmpstr[28] = 0; // overwrite '0'
+
+  // Can't use regular addMetadata, because of the zero term after the namespace id; and the APP1 marker
+  jpeg_write_marker( &cinfo, JPEG_APP0+1, (const JOCTET*) xmpstr, 29 + xmp.size() );
 }
