@@ -1,6 +1,6 @@
 /*  JPEG class wrapper to ijg jpeg library
 
-    Copyright (C) 2000-2012 Ruven Pillay.
+    Copyright (C) 2000-2018 Ruven Pillay.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -23,10 +23,13 @@
 #define _JPEGCOMPRESSOR_H
 
 
+#include "Compressor.h"
 
-#include <cstdio>
-#include <string>
-#include "RawTile.h"
+
+// Fix missing snprintf in Windows
+#if _MSC_VER
+#define snprintf _snprintf
+#endif
 
 
 extern "C"{
@@ -57,8 +60,8 @@ typedef iip_destination_mgr * iip_dest_ptr;
 
 /// Wrapper class to the IJG JPEG library
 
-class JPEGCompressor{
-	
+class JPEGCompressor: public Compressor{
+
  private:
 
   /// the width, height and number of channels per sample for the image
@@ -73,7 +76,7 @@ class JPEGCompressor{
   /// Buffer for the image data
   unsigned char *data;
 
-  /// Size of the JPEG header 
+  /// Size of the JPEG header
   unsigned int header_size;
 
   /// JPEG library objects
@@ -82,17 +85,23 @@ class JPEGCompressor{
   iip_destination_mgr dest_mgr;
   iip_dest_ptr dest;
 
+  /// Write ICC profile
+  void writeICCProfile();
+
+  /// Write XMP metadata
+  void writeXMPMetadata();
+
 
  public:
 
   /// Constructor
   /** @param quality JPEG Quality factor (0-100) */
-   JPEGCompressor( int quality ) { Q = quality; dest = NULL; };
+  JPEGCompressor( int quality ) { Q = quality; dest = NULL; };
 
 
   /// Set the compression quality
   /** @param factor Quality factor (0-100) */
-  void setQuality( int factor ) {
+  inline void setQuality( int factor ) {
     if( factor < 0 ) Q = 0;
     else if( factor > 100 ) Q = 100;
     else Q = factor;
@@ -100,7 +109,7 @@ class JPEGCompressor{
 
 
   /// Get the current quality level
-  int getQuality() { return Q; }
+  inline int getQuality() { return Q; }
 
 
   /// Initialise strip based compression
@@ -111,38 +120,36 @@ class JPEGCompressor{
       @param strip_height pixel height of the strip we want to compress
       @return header size
    */
-  void InitCompression( const RawTile& rawtile, unsigned int strip_height ) throw (std::string);
+  void InitCompression( const RawTile& rawtile, unsigned int strip_height );
 
   /// Compress a strip of image data
   /** @param s source image data
       @param o output buffer
       @param tile_height pixel height of the tile we are compressing
    */
-  unsigned int CompressStrip( unsigned char* s, unsigned char* o, unsigned int tile_height ) throw (std::string);
+  unsigned int CompressStrip( unsigned char* s, unsigned char* o, unsigned int tile_height );
 
   /// Finish the strip based compression and free memory
   /** @param output output buffer
       @return size of output generated
    */
-  unsigned int Finish( unsigned char* output ) throw (std::string);
-
+  unsigned int Finish( unsigned char* output );
 
   /// Compress an entire buffer of image data at once in one command
   /** @param t tile of image data */
-  int Compress( RawTile& t ) throw (std::string);
-
-
-  /// Add metadata to the JPEG header
-  /** @param m metadata */
-  void addMetadata( const std::string& m );
-
+  unsigned int Compress( RawTile& t );
 
   /// Return the JPEG header size
-  unsigned int getHeaderSize() { return header_size; }
+  inline unsigned int getHeaderSize() { return header_size; }
 
   /// Return a pointer to the header itself
   inline unsigned char* getHeader() { return header; }
 
+  /// Return the JPEG mime type
+  inline const char* getMimeType(){ return "image/jpeg"; }
+
+  /// Return the image filename suffix
+  inline const char* getSuffix(){ return "jpg"; }
 
 };
 
