@@ -9,7 +9,7 @@
     Culture of the Czech Republic.
 
 
-    Copyright (C) 2009-2016 IIPImage.
+    Copyright (C) 2009-2017 IIPImage.
     Author: Ruven Pillay
 
     This program is free software; you can redistribute it and/or modify
@@ -33,6 +33,7 @@
 
 
 #include "IIPImage.h"
+#include "Logger.h"
 
 #include <jpx.h>
 #include <jp2.h>
@@ -46,7 +47,7 @@
 using namespace kdu_supp; // Also includes the `kdu_core' namespace
 #endif
 
-extern std::ofstream logfile;
+extern Logger logfile;
 
 
 /// Wrapper class to handle error messages from Kakadu
@@ -115,7 +116,7 @@ class KakaduImage : public IIPImage {
       @param h height of region
       @param d buffer to fill
    */
-  void process( unsigned int r, int l, int x, int y, unsigned int w, unsigned int h, void* d ) throw (file_error);
+  void process( unsigned int r, int l, int x, int y, unsigned int w, unsigned int h, void* d );
 
   /// Convenience function to delete allocated buffers
   /** @param b pointer to buffer
@@ -150,7 +151,7 @@ class KakaduImage : public IIPImage {
   };
 
   /// Assignment Operator
-  /** @param TPTImage object
+  /** @param image object
    */
   KakaduImage& operator = ( KakaduImage image ) {
     if( this != &image ){
@@ -164,15 +165,15 @@ class KakaduImage : public IIPImage {
   /// Destructor
   ~KakaduImage() { closeImage(); };
 
-  /// Overloaded function for opening a TIFF image
-  void openImage() throw (file_error);
 
+  /// Overloaded function for opening a TIFF image
+  void openImage();
 
   /// Overloaded function for loading TIFF image information
   /** @param x horizontal sequence angle
       @param y vertical sequence angle
    */
-  void loadImageInfo( int x, int y ) throw (file_error);
+  void loadImageInfo( int x, int y );
 
   /// Overloaded function for closing a JPEG2000 image
   void closeImage();
@@ -187,7 +188,7 @@ class KakaduImage : public IIPImage {
       @param l number of quality layers to decode
       @param t tile number
    */
-  RawTile getTile( int x, int y, unsigned int r, int l, unsigned int t ) throw (file_error);
+  RawTile getTile( int x, int y, unsigned int r, int l, unsigned int t );
 
   /// Overloaded function for returning a region for a given angle and resolution
   /** Return a RawTile object: Overloaded by child class.
@@ -199,9 +200,18 @@ class KakaduImage : public IIPImage {
       @param y y coordinate
       @param w width of region
       @param h height of region
-      @param b buffer to fill
+      @return RawTile image
    */
-  RawTile getRegion( int ha, int va, unsigned int r, int l, int x, int y, unsigned int w, unsigned int h ) throw (file_error);
+  RawTile getRegion( int ha, int va, unsigned int r, int l, int x, int y, unsigned int w, unsigned int h );
+
+  /// Read-mode types
+  enum KDU_READMODE { KDU_FAST,     ///< Default fast mode
+		      KDU_FUSSY,    ///< Fussy mode
+		      KDU_RESILIENT ///< Reslient mode for damaged JP2 streams
+  };
+
+  /// Read-mode
+  KDU_READMODE kdu_readmode;
 
 
 };
