@@ -2,7 +2,7 @@
 
 /*  IIP Image Server
 
-    Copyright (C) 2000-2016 Ruven Pillay.
+    Copyright (C) 2000-2019 Ruven Pillay.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -31,7 +31,7 @@
 
 
 /// Colour spaces - GREYSCALE, sRGB and CIELAB
-enum ColourSpaces { NONE, GREYSCALE, sRGB, CIELAB };
+enum ColourSpaces { NONE, GREYSCALE, sRGB, CIELAB, BINARY };
 
 /// Compression Types
 enum CompressionType { UNCOMPRESSED, JPEG, DEFLATE, PNG };
@@ -79,7 +79,7 @@ class RawTile{
   int memoryManaged;
 
   /// The size of the data pointed to by data
-  int dataLength;
+  unsigned int dataLength;
 
   /// The width in pixels of this tile
   unsigned int width;
@@ -221,7 +221,7 @@ class RawTile{
 
 
   /// Return the size of the data
-  int size() { return dataLength; }
+  unsigned int size() { return dataLength; }
 
 
   /// Overloaded equality operator
