@@ -1,7 +1,7 @@
 /*
     IIP Command Handler Member Functions
 
-    Copyright (C) 2006-2014 Ruven Pillay.
+    Copyright (C) 2006-2019 Ruven Pillay.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -63,6 +63,7 @@ Task* Task::factory( const string& t ){
   else if( type == "lyr" ) return new LYR;
   else if( type == "deepzoom" ) return new DeepZoom;
   else if( type == "ctw" ) return new CTW;
+  else if( type == "col" ) return new COL;
   else if( type == "iiif" ) return new IIIF;
   else return NULL;
 
@@ -122,10 +123,18 @@ void MINMAX::run( Session* session, const string& argument ){
 
   if( session->loglevel >= 3 ) *(session->logfile) << "MINMAX handler reached" << endl;
 
-  // Parse the argument list
+  // Parse the argument list: command is of the form MINXMAX=<channel>:<min>,<max>
   int delimitter = argument.find( ":" );
   string tmp = argument.substr( 0, delimitter );
   int nchan = atoi( tmp.c_str() ) - 1;
+
+  // Sanity check for channel index
+  if( nchan < 0 || nchan > (int)(*session->image)->getNumChannels() ){
+    if( session->loglevel >= 1 ) *(session->logfile) << "MINMAX :: Error: channel number out of bounds: "
+						     << tmp.c_str() << endl;
+    return;
+  }
+
   string arg2 = argument.substr( delimitter + 1, argument.length() );
 
   delimitter = arg2.find( "," );
@@ -144,12 +153,27 @@ void MINMAX::run( Session* session, const string& argument ){
 
 void CNT::run( Session* session, const string& argument ){
 
-  float contrast = (float) atof( argument.c_str() );
-
   if( session->loglevel >= 2 ) *(session->logfile) << "CNT handler reached" << endl;
-  if( session->loglevel >= 3 ) *(session->logfile) << "CNT :: requested contrast adjustment is " << contrast << endl;
 
-  session->view->setContrast( contrast );
+  // Request for histogram equalization
+  string arg = argument;
+  transform( arg.begin(), arg.end(), arg.begin(), ::tolower );
+  if( arg == "eq" || arg == "equalization" ){
+    session->view->equalization = true;
+    if( session->loglevel >= 3 ) *(session->logfile) << "CNT :: histogram equalization requested" << endl;
+  }
+  // Linear stretch
+  else if( arg == "st" || arg == "stretch" ){
+    // Use reserved value of -1 for contrast stretch
+    session->view->contrast = -1;
+    if( session->loglevel >= 3 ) *(session->logfile) << "CNT :: contrast stretch requested" << endl;
+  }
+  // Contrast adjustment by pixel multiplication
+  else{
+    float contrast = (float) atof( argument.c_str() );
+    session->view->contrast = contrast;
+    if( session->loglevel >= 3 ) *(session->logfile) << "CNT :: requested contrast adjustment is " << contrast << endl;
+  }
 }
 
 
@@ -160,7 +184,7 @@ void GAM::run( Session* session, const string& argument ){
   if( session->loglevel >= 2 ) *(session->logfile) << "GAM handler reached" << endl;
   if( session->loglevel >= 3 ) *(session->logfile) << "GAM :: requested gamma adjustment is " << gamma << endl;
 
-  session->view->setGamma( gamma );
+  session->view->gamma = gamma;
 }
 
 
@@ -176,6 +200,7 @@ void CVT::run( Session* session, const string& src ){
     if( session->loglevel >= 1 ) *(session->logfile) << "CVT :: Unsupported request: '" << argument << "'. Sending JPEG." << endl;
   }
   else{
+    session->view->output_format = JPEG;
     if( session->loglevel >= 3 ) *(session->logfile) << "CVT :: JPEG output" << endl;
   }
 
@@ -253,7 +278,9 @@ void ROT::run( Session* session, const string& argument ){
   if( session->loglevel >= 2 ) *(session->logfile) << "ROT handler reached" << endl;
   if( session->loglevel >= 3 ) *(session->logfile) << "ROT :: requested rotation is " << rotation << " degrees" << endl;
 
-  session->view->setRotation( rotation );
+  // Set rotation - watch for a '!180' request, which is simply a vertical flip
+  if( session->view->flip == 1 && rotation == 180 ) session->view->flip = 2;
+  else session->view->setRotation( rotation );
 }
 
 
@@ -452,3 +479,21 @@ void CTW::run( Session* session, const string& argument ){
   }
 
 }
+
+
+void COL::run( Session* session, const string& argument ){
+  /* The argument is the output color conversion. Supported values:
+     GREY/GRAY: grayscale, BINARY: binary (bilevel)
+  */
+
+  // Convert to lower case in order to do our string comparison
+  string ctype = argument;
+  transform( ctype.begin(), ctype.end(), ctype.begin(), ::tolower );
+
+  if( session->loglevel >= 2 ) *(session->logfile) << "COL handler reached" << endl;
+  if( session->loglevel >= 3 ) *(session->logfile) << "COL :: requested color transform to " << ctype << endl;
+
+  if( ctype == "grey" || ctype == "gray" ) session->view->colourspace = GREYSCALE;
+  else if( ctype == "binary" ) session->view->colourspace = BINARY;
+  
+}
