@@ -1,8 +1,8 @@
 // Image Transform Functions
 
-/*  IIP fcgi server module - image processing routines
+/*  IIPImage image processing routines
 
-    Copyright (C) 2004-2016 Ruven Pillay.
+    Copyright (C) 2004-2019 Ruven Pillay.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -21,6 +21,7 @@
 
 
 #include <cmath>
+#include <algorithm>
 #include "Transforms.h"
 
 
@@ -31,7 +32,7 @@
 #include <limits>
 static bool isfinite( float arg )
 {
-  return arg == arg && 
+  return arg == arg &&
     arg != std::numeric_limits<float>::infinity() &&
     arg != -std::numeric_limits<float>::infinity();
 }
@@ -59,7 +60,7 @@ using namespace std;
 
 
 // Normalization function
-void filter_normalize( RawTile& in, vector<float>& max, vector<float>& min ) {
+void Transform::normalize( RawTile& in, const vector<float>& max, const vector<float>& min ) {
 
   float *normdata;
   unsigned int np = in.dataLength * 8 / in.bpc;
@@ -149,14 +150,14 @@ void filter_normalize( RawTile& in, vector<float>& max, vector<float>& min ) {
   // Assign our new buffer and modify some info
   in.data = normdata;
   in.bpc = 32;
-  in.dataLength = np * in.bpc / 8;
+  in.dataLength = np * (in.bpc/8);
 
 }
 
 
 
 // Hillshading function
-void filter_shade( RawTile& in, int h_angle, int v_angle ){
+void Transform::shade( RawTile& in, int h_angle, int v_angle ){
 
   float o_x, o_y, o_z;
 
@@ -221,13 +222,13 @@ void filter_shade( RawTile& in, int h_angle, int v_angle ){
 
   in.data = buffer;
   in.channels = 1;
-  in.dataLength = in.width * in.height * in.bpc / 8;
+  in.dataLength = in.width * in.height * (in.bpc/8);
 }
 
 
 
 // Convert a single pixel from CIELAB to sRGB
-static void LAB2sRGB( unsigned char *in, unsigned char *out ){
+void Transform::LAB2sRGB( unsigned char *in, unsigned char *out ){
 
   /* First convert to XYZ
    */
@@ -269,7 +270,6 @@ static void LAB2sRGB( unsigned char *in, unsigned char *out ){
   Y /= 100.0;
   Z /= 100.0;
 
-
   /* Then convert to sRGB
    */
   R = (X * _sRGB[0][0]) + (Y * _sRGB[0][1]) + (Z * _sRGB[0][2]);
@@ -278,10 +278,9 @@ static void LAB2sRGB( unsigned char *in, unsigned char *out ){
 
   /* Clip any -ve values
    */
-  if( R < 0.0 ) R = 0.0;
-  if( G < 0.0 ) G = 0.0;
-  if( B < 0.0 ) B = 0.0;
-
+  R = (R<0.0 ? 0.0 : R);
+  G = (G<0.0 ? 0.0 : G);
+  B = (B<0.0 ? 0.0 : B);
 
   /* We now need to convert these to non-linear display values
    */
@@ -302,10 +301,9 @@ static void LAB2sRGB( unsigned char *in, unsigned char *out ){
 
   /* Clip to our 8 bit limit
    */
-  if( R > 255.0 ) R = 255.0;
-  if( G > 255.0 ) G = 255.0;
-  if( B > 255.0 ) B = 255.0;
-
+  R = (R>255.0 ? 255.0 : R);
+  G = (G>255.0 ? 255.0 : G);
+  B = (B>255.0 ? 255.0 : B);
 
   /* Return our sRGB values
    */
@@ -318,7 +316,7 @@ static void LAB2sRGB( unsigned char *in, unsigned char *out ){
 
 
 // Convert whole tile from CIELAB to sRGB
-void filter_LAB2sRGB( RawTile& in ){
+void Transform::LAB2sRGB( RawTile& in ){
 
   unsigned long np = in.width * in.height * in.channels;
 
@@ -341,7 +339,9 @@ void filter_LAB2sRGB( RawTile& in ){
 
 
 // Colormap function
-void filter_cmap( RawTile& in, enum cmap_type cmap ){
+// Based on the routine colormap.cpp in Imagin Raytracer by Olivier Ferrand
+// http://www.imagin-raytracer.org
+void Transform::cmap( RawTile& in, enum cmap_type cmap ){
 
   float value;
   unsigned in_chan = in.channels;
@@ -356,6 +356,7 @@ void filter_cmap( RawTile& in, enum cmap_type cmap ){
   float *outv = outptr;
 
   switch(cmap){
+
     case HOT:
 #if defined(__ICC) || defined(__INTEL_COMPILER)
 #pragma ivdep
@@ -375,6 +376,7 @@ void filter_cmap( RawTile& in, enum cmap_type cmap ){
         else { outv[0]=outv[1]=outv[2]=1.; }
       }
       break;
+
     case COLD:
 #if defined(__ICC) || defined(__INTEL_COMPILER)
 #pragma ivdep
@@ -394,6 +396,7 @@ void filter_cmap( RawTile& in, enum cmap_type cmap ){
         else {outv[0]=outv[1]=outv[2]=1.;}
       }
       break;
+
     case JET:
 #if defined(__ICC) || defined(__INTEL_COMPILER)
 #pragma ivdep
@@ -415,22 +418,56 @@ void filter_cmap( RawTile& in, enum cmap_type cmap ){
         else { outv[0]=0.5; outv[1]=outv[2]=0.; }
       }
       break;
+
+    case RED:
+#if defined(__ICC) || defined(__INTEL_COMPILER)
+#pragma ivdep
+#endif
+      for( unsigned int n=0; n<ndata; n+=in_chan, outv+=3 ){
+	value = fptr[n];
+	outv[0] = value;
+	outv[1] = outv[2] = 0.;
+      }
+      break;
+
+    case GREEN:
+#if defined(__ICC) || defined(__INTEL_COMPILER)
+#pragma ivdep
+#endif
+      for( unsigned int n=0; n<ndata; n+=in_chan, outv+=3 ) {
+	value = fptr[n];
+	outv[0] = outv[2] = 0.;
+	outv[1] = value;
+      }
+      break;
+
+    case BLUE:
+#if defined(__ICC) || defined(__INTEL_COMPILER)
+#pragma ivdep
+#endif
+      for( unsigned int n=0; n<ndata; n+=in_chan, outv+=3 ) {
+	value = fptr[n];
+	outv[0] = outv[1] = 0;
+	outv[2] = value;
+      }
+      break;
+
     default:
       break;
-    };
 
+  };
 
   // Delete old data buffer
   delete[] (float*) in.data;
   in.data = outptr;
   in.channels = out_chan;
-  in.dataLength = ndata * out_chan * in.bpc / 8;
+  in.dataLength = ndata * out_chan * (in.bpc/8);
 }
 
 
 
 // Inversion function
-void filter_inv( RawTile& in ){
+void Transform::inv( RawTile& in ){
 
   unsigned int np = in.dataLength * 8 / in.bpc;
   float *infptr = (float*) in.data;
@@ -450,7 +487,7 @@ void filter_inv( RawTile& in ){
 
 
 // Resize image using nearest neighbour interpolation
-void filter_interpolate_nearestneighbour( RawTile& in, unsigned int resampled_width, unsigned int resampled_height ){
+void Transform::interpolate_nearestneighbour( RawTile& in, unsigned int resampled_width, unsigned int resampled_height ){
 
   // Pointer to input buffer
   unsigned char *input = (unsigned char*) in.data;
@@ -496,7 +533,7 @@ void filter_interpolate_nearestneighbour( RawTile& in, unsigned int resampled_wi
   // Correctly set our Rawtile info
   in.width = resampled_width;
   in.height = resampled_height;
-  in.dataLength = resampled_width * resampled_height * channels * in.bpc/8;
+  in.dataLength = resampled_width * resampled_height * channels * (in.bpc/8);
   in.data = output;
 }
 
@@ -504,7 +541,7 @@ void filter_interpolate_nearestneighbour( RawTile& in, unsigned int resampled_wi
 
 // Resize image using bilinear interpolation
 //  - Floating point implementation which benchmarks about 2.5x slower than nearest neighbour
-void filter_interpolate_bilinear( RawTile& in, unsigned int resampled_width, unsigned int resampled_height ){
+void Transform::interpolate_bilinear( RawTile& in, unsigned int resampled_width, unsigned int resampled_height ){
 
   // Pointer to input buffer
   unsigned char *input = (unsigned char*) in.data;
@@ -512,13 +549,14 @@ void filter_interpolate_bilinear( RawTile& in, unsigned int resampled_width, uns
   int channels = in.channels;
   unsigned int width = in.width;
   unsigned int height = in.height;
+  unsigned long np = in.channels * in.width * in.height;
 
   // Create new buffer and pointer for our output
   unsigned char *output = new unsigned char[resampled_width*resampled_height*in.channels];
 
   // Calculate our scale
-  float xscale = (float)(width-1) / (float)resampled_width;
-  float yscale = (float)(height-1) / (float)resampled_height;
+  float xscale = (float)(width) / (float)resampled_width;
+  float yscale = (float)(height) / (float)resampled_height;
 
 
   // Do not parallelize for small images (256x256 pixels) as this can be slower that single threaded
@@ -544,10 +582,16 @@ void filter_interpolate_bilinear( RawTile& in, unsigned int resampled_width, uns
 
       // Calculate the indices of the 4 surrounding pixels
       unsigned int p11, p12, p21, p22;
-      p11 = (unsigned int) ( channels * ( ii + jj*width ) );
-      p12 = (unsigned int) ( channels * ( ii + (jj+1)*width ) );
-      p21 = (unsigned int) ( channels * ( (ii+1) + jj*width ) );
-      p22 = (unsigned int) ( channels * ( (ii+1) + (jj+1)*width ) );
+      unsigned long jj_w = jj*width;
+      p11 = (unsigned int) ( channels * ( ii + jj_w ) );
+      p12 = (unsigned int) ( channels * ( ii + (jj_w+width) ) );
+      p21 = (unsigned int) ( channels * ( (ii+1) + jj_w ) );
+      p22 = (unsigned int) ( channels * ( (ii+1) + (jj_w+width) ) );
+
+      // Make sure we don't stray outside our input buffer boundary
+      // - use replication at the edge
+      p12 = (p12<=np)? p12 : np-channels;
+      p22 = (p22<=np)? p22 : np-channels;
 
       // Calculate the rest of our weights
       float iscale = i*xscale;
@@ -572,16 +616,16 @@ void filter_interpolate_bilinear( RawTile& in, unsigned int resampled_width, uns
   // Correctly set our Rawtile info
   in.width = resampled_width;
   in.height = resampled_height;
-  in.dataLength = resampled_width * resampled_height * channels * in.bpc/8;
+  in.dataLength = resampled_width * resampled_height * channels * (in.bpc/8);
   in.data = output;
 }
 
 
 
 // Function to apply a contrast adjustment and clip to 8 bit
-void filter_contrast( RawTile& in, float c ){
+void Transform::contrast( RawTile& in, float c ){
 
-  unsigned int np = in.dataLength * 8 / in.bpc;
+  unsigned long np = in.width * in.height * in.channels;
   unsigned char* buffer = new unsigned char[np];
   float* infptr = (float*)in.data;
 
@@ -590,7 +634,7 @@ void filter_contrast( RawTile& in, float c ){
 #elif defined(_OPENMP)
 #pragma omp parallel for
 #endif
-  for( unsigned int n=0; n<np; n++ ){
+  for( unsigned long n=0; n<np; n++ ){
     float v = infptr[n] * 255.0 * c;
     buffer[n] = (unsigned char)( (v<255.0) ? (v<0.0? 0.0 : v) : 255.0 );
   }
@@ -599,13 +643,13 @@ void filter_contrast( RawTile& in, float c ){
   delete[] (float*) in.data;
   in.data = buffer;
   in.bpc = 8;
-  in.dataLength = np * in.bpc/8;
+  in.dataLength = np * (in.bpc/8);
 }
 
 
 
 // Gamma correction
-void filter_gamma( RawTile& in, float g ){
+void Transform::gamma( RawTile& in, float g ){
 
   if( g == 1.0 ) return;
 
@@ -627,12 +671,12 @@ void filter_gamma( RawTile& in, float g ){
 
 
 // Rotation function
-void filter_rotate( RawTile& in, float angle=0.0 ){
+void Transform::rotate( RawTile& in, float angle=0.0 ){
 
   // Currently implemented only for rectangular rotations
   if( (int)angle % 90 == 0 && (int)angle % 360 != 0 ){
 
-    // Intialize our counter
+    // Initialize our counter
     unsigned int n = 0;
 
     // Allocate memory for our temporary buffer - rotate function only ever operates on 8bit data
@@ -664,8 +708,8 @@ void filter_rotate( RawTile& in, float angle=0.0 ){
 #pragma omp parallel for if( in.width*in.height > PARALLEL_THRESHOLD )
 #endif
       for( int i=in.width-1; i>=0; i-- ){
-	unsigned int n = i*in.height*in.channels;
-	for( unsigned int j=0; j < in.height; j++ ){
+	unsigned int n = (in.width-1-i)*in.height*in.channels;
+	for( unsigned int j=0; j<in.height; j++ ){
 	  unsigned int index = (in.width*j + i)*in.channels;
 	  for( int k=0; k < in.channels; k++ ){
 	    ((unsigned char*)buffer)[n++] = ((unsigned char*)in.data)[index+k];
@@ -704,7 +748,7 @@ void filter_rotate( RawTile& in, float angle=0.0 ){
 // Convert colour to grayscale using the conversion formula:
 //   Luminance = 0.2126*R + 0.7152*G + 0.0722*B
 // Note that we don't linearize before converting
-void filter_greyscale( RawTile& rawtile ){
+void Transform::greyscale( RawTile& rawtile ){
 
   if( rawtile.bpc != 8 || rawtile.channels != 3 ) return;
 
@@ -738,7 +782,7 @@ void filter_greyscale( RawTile& rawtile ){
 
 
 // Apply twist or channel recombination to colour or multi-channel image
-void filter_twist( RawTile& rawtile, const vector< vector<float> >& matrix ){
+void Transform::twist( RawTile& rawtile, const vector< vector<float> >& matrix ){
 
   unsigned long np = rawtile.width * rawtile.height;
 
@@ -785,7 +829,7 @@ void filter_twist( RawTile& rawtile, const vector< vector<float> >& matrix ){
 
 // Flatten a multi-channel image to a given number of bands by simply stripping
 // away extra bands
-void filter_flatten( RawTile& in, int bands ){
+void Transform::flatten( RawTile& in, int bands ){
 
   // We cannot increase the number of channels
   if( bands >= in.channels ) return;
@@ -804,14 +848,13 @@ void filter_flatten( RawTile& in, int bands ){
   }
 
   in.channels = bands;
-  in.dataLength = ni * in.bpc/8;
+  in.dataLength = ni * (in.bpc/8);
 }
 
 
 
-
 // Flip image in horizontal or vertical direction (0=horizontal,1=vertical)
-void filter_flip( RawTile& rawtile, int orientation ){
+void Transform::flip( RawTile& rawtile, int orientation ){
 
   unsigned char* buffer = new unsigned char[rawtile.width * rawtile.height * rawtile.channels];
 
@@ -823,7 +866,7 @@ void filter_flip( RawTile& rawtile, int orientation ){
 #pragma omp parallel for if( rawtile.width*rawtile.height > PARALLEL_THRESHOLD )
 #endif
     for( int j=rawtile.height-1; j>=0; j-- ){
-      unsigned long n = j*rawtile.width*rawtile.channels;
+      unsigned long n = (rawtile.height-1-j)*rawtile.width*rawtile.channels;
       for( unsigned int i=0; i<rawtile.width; i++ ){
         unsigned long index = (rawtile.width*j + i)*rawtile.channels;
         for( int k=0; k<rawtile.channels; k++ ){
@@ -854,3 +897,148 @@ void filter_flip( RawTile& rawtile, int orientation ){
   delete[] (unsigned char*) rawtile.data;
   rawtile.data = (void*) buffer;
 }
+
+
+
+// Calculate histogram of an image
+//  - Only calculate for 8 bits and a single histogram for all channels
+vector<unsigned int> Transform::histogram( RawTile& in, const vector<float>& max, const vector<float>& min ){
+
+  // An 8 bit (256 level) histogram should be sufficient
+  if( in.bpc > 8 ){
+    this->normalize( in, max, min );
+    this->contrast( in, 1.0 );
+  }
+
+  // Initialize our vector to zero - note that we use a single histogram for all channels
+  vector<unsigned int> histogram( (1<<in.bpc), 0 );
+
+  // Fill our histogram - for color or multiband images, use channel average
+  unsigned int np = in.width * in.height;
+  for( unsigned int n=0; n<np; n++ ){
+    float value = 0.0;
+
+    // For color or multiband images, use channel average
+    for( int k=0; k<in.channels; k++ ){
+      value += (float)(((unsigned char*)in.data)[n*in.channels + k]);
+    }
+    value = round( value/(float)in.channels );
+
+    // Update histogram
+    histogram[(unsigned int)value]++;
+  }
+
+  return histogram;
+}
+
+
+
+// Calculate the threshold for binary image segmentation using Otsu's method
+unsigned char Transform::threshold( vector<unsigned int>& histogram ){
+
+  const unsigned int bits = histogram.size();
+
+  // Calculate sum
+  float sum = 0.0, sumb = 0.0;
+  unsigned int np = 0;
+  for( unsigned int n=0; n<bits; n++ ){
+    np += histogram[n];
+    sum += (float)n * histogram[n];
+  }
+
+  // Calculate threshold
+  float wb = 0.0, wf = 0.0, mb = 0.0, mf = 0.0, max = 0.0;
+  unsigned char otsu = 0;
+  for( unsigned int n=0; n<bits; n++ ){
+    wb += histogram[n];
+    if( wb == 0.0 ) continue;
+
+    wf = np - wb;
+    if( wf == 0.0 ) break;
+
+    sumb += (float) n * histogram[n];
+    mb = sumb / wb;
+    mf = (sum-sumb) / wf;
+    float diff = wb * wf * (mb-mf) * (mb-mf);
+
+    if( diff > max ){
+      otsu = (unsigned char) n;
+      max = diff;
+    }
+  }
+  return otsu;
+}
+
+
+
+// Apply threshold to create binary image
+void Transform::binary( RawTile &in, unsigned char threshold ){
+
+  // Only apply to 8 bit images
+  if( in.bpc != 8 ) return;
+
+  // First make sure our image is greyscale
+  this->greyscale( in );
+
+  unsigned int np = in.width * in.height;
+
+#if defined(__ICC) || defined(__INTEL_COMPILER)
+#pragma ivdep
+#elif defined(_OPENMP)
+#pragma omp parallel for if( in.width*in.height > PARALLEL_THRESHOLD )
+#endif
+  for( unsigned int i=0; i<np; i++ ){
+    ((unsigned char*)in.data)[i] = ( ((unsigned char*)in.data)[i] < threshold ? (unsigned char)0 : (unsigned char)255 );
+  }
+}
+
+
+
+void Transform::equalize( RawTile& in, vector<unsigned int>& histogram ){
+
+  // Number of levels in our histogram
+  const unsigned int bits = histogram.size();
+
+  // Allocate and initialize our array to zero using std::fill
+  float *cdf = new float[bits];
+  fill( cdf, cdf+bits, 0.0 );
+
+  // Find first non-zero bin
+  unsigned int n0 = 0;
+  while( histogram[n0] == 0 ) ++n0;
+
+  // Calculate cumulative histogram
+  cdf[0] = histogram[0];
+  for( unsigned int i=1; i<bits; i++ ){
+    cdf[i] = cdf[i-1] + histogram[i];
+  }
+
+  // Scale our CDF
+  float scale = (float)(bits-1) / cdf[bits-1];
+  float cdfmin = cdf[n0] / (float)(in.width*in.height);
+#if defined(__ICC) || defined(__INTEL_COMPILER)
+#pragma ivdep
+#elif defined(_OPENMP)
+#pragma omp parallel for
+#endif
+  for( unsigned int i=0; i<bits; i++ ){
+    cdf[i] = round( scale * (cdf[i]-cdfmin) );
+  }
+
+  // Map image through cumulative histogram
+#if defined(__ICC) || defined(__INTEL_COMPILER)
+#pragma ivdep
+#elif defined(_OPENMP)
+#pragma omp parallel for if( in.width*in.height > PARALLEL_THRESHOLD )
+#endif
+  for( unsigned int i=0; i<in.width*in.height; i++ ){
+    for( int j=0; j<in.channels; j++ ){
+      unsigned int index = i*in.channels + j;
+      unsigned int value = (unsigned int) (((unsigned char*)in.data)[index]);
+      ((unsigned char*)in.data)[index] = (unsigned char) cdf[value];
+    }
+  }
+
+  // Free our dynamically allocated array
+  delete[] cdf;
+}
