@@ -1,7 +1,7 @@
 /*
     Image View and Transform Parameters
 
-    Copyright (C) 2003-2016 Ruven Pillay.
+    Copyright (C) 2003-2019 Ruven Pillay.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -28,11 +28,6 @@
 
 #include "Transforms.h"
 
-//include round function for MSVC compiler
-#if _MSC_VER
-#include "../windows/Time.h"
-#endif
-
 
 
 
@@ -54,8 +49,6 @@ class View{
   unsigned int max_size;                      /// Maximum viewport dimension
   unsigned int requested_width;               /// Width requested by WID command
   unsigned int requested_height;              /// Height requested by HEI command
-  float contrast;                             /// Contrast adjustment requested by CNT command
-  float gamma;                                /// Gamma adjustment requested by GAM command
   float rotation;                             /// Rotation requested by ROT command
 
 
@@ -82,6 +75,12 @@ class View{
   std::vector< std::vector<float> > ctw;      /// Colour twist matrix
   int flip;                                   /// Flip (1=horizontal, 2=vertical)
   bool maintain_aspect;                       /// Indicate whether aspect ratio should be maintained
+  bool allow_upscaling;                       /// Indicate whether images may be served larger than the source file
+  bool embed_icc;                             /// Indicate whether we should embed ICC profiles
+  CompressionType output_format;              /// Requested output format
+  float contrast;                             /// Contrast adjustment requested by CNT command
+  float gamma;                                /// Gamma adjustment requested by GAM command
+  bool equalization;                          /// Whether to perform histogram equalization
 
 
   /// Constructor
@@ -99,15 +98,14 @@ class View{
     max_layers = 0; layers = 0;
     rotation = 0.0; flip = 0;
     maintain_aspect = true;
+    allow_upscaling = true;
     colourspace = NONE;
+    embed_icc = true;
+    output_format = JPEG;
+    equalization = false;
   };
 
 
-  /// Set the contrast adjustment
-  /** @param c contrast (where 1.0 is no adjustment) */
-  void setContrast( float c ){ contrast = c; };
-
-
   /// Set the maximum view port dimension
   /** @param m maximum viewport dimension */
   void setMaxSize( unsigned int m ){ max_size = m; };
@@ -116,6 +114,32 @@ class View{
   /// Get the maximum allowed output size
   /* @return maximum output dimension */
   unsigned int getMaxSize(){ return max_size; };
+  
+
+  /// Set the allow_upscaling flag
+  /** @param upscale allow upscaling of source image */
+  void setAllowUpscaling( bool upscale ){ allow_upscaling = upscale; };
+
+
+  /// Get the allow_upscaling flag
+  /* @return true or false */
+  bool allowUpscaling(){ return allow_upscaling; };
+
+
+  /// Set the embed_icc flag
+  /** @param embed embed icc profile flag
+   */
+  void setEmbedICC( bool embed ){ embed_icc = embed; };
+
+
+  /// Get the embed_icc flag - disable in case of certain types of processing
+  /** @return whether ICC profile should be embedded
+   */
+  bool embedICC(){
+    // Disable if colour-mapping, twist, hill-shading or greyscale conversion applied
+    if( cmapped || shaded || ctw.size() || colourspace==GREYSCALE ) return false;
+    return embed_icc;
+  };
 
 
   /// Set the maximum view port dimension
@@ -131,8 +155,7 @@ class View{
   /// Set the size of the requested width
   /** @param w requested image width */
   void setRequestWidth( unsigned int w ){
-    if( (max_size > 0) && (w > max_size) ) requested_width = max_size;
-    else requested_width = w;
+    requested_width = w;
   };
 
 
@@ -144,8 +167,7 @@ class View{
   /// Set the size of the requested height
   /** @param h requested image height */
   void setRequestHeight( unsigned int h ){
-    if( (max_size > 0) && (h > max_size) ) requested_height = max_size;
-    else requested_height = h;
+    requested_height = h;
   };
 
 
@@ -197,10 +219,6 @@ class View{
   /// Return the number of layers to decode
   int getLayers();
 
-  /// Return the contrast adjustment
-  /* @return requested contrast */
-  float getContrast(){ return contrast; };
-
   /// Return the image width at our requested resolution
   /* @return image width */
   unsigned int getImageWidth(){ return width; };
@@ -229,14 +247,6 @@ class View{
   /* @return boolean indicating whether viewport specified */
   bool viewPortSet();
 
-  /// Set gamma
-  /** @param g gamma value */
-  void setGamma( float g ){ gamma = g; };
-
-  /// Get gamma
-  /* @return requested gamma */
-  float getGamma(){ return gamma; };
-
   /// Set rotation
   /** @param r angle of rotation in degrees */
   void setRotation( float r ){ rotation = r; };
@@ -253,6 +263,12 @@ class View{
     else return false;
   }
 
+  /// Whether we require a histogram
+  bool requireHistogram(){
+    if( equalization || colourspace==BINARY || contrast==-1 ) return true;
+    else return false;
+  }
+
 };
 
 
